/*
 * Copyright 2002 Niels Provos <provos@citi.umich.edu>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Niels Provos.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _IPFRAG_H_
#define _IPFRAG_H_

struct fragent {
	TAILQ_ENTRY(fragent) next;
	u_short off;
	u_short len;
	u_short size;
	u_char *data;
};

struct fragment {
	SPLAY_ENTRY(fragment) node;
	TAILQ_ENTRY(fragment) next;

	TAILQ_HEAD(fragq, fragent) fraglist;
	struct event timeout;

	enum fragpolicy fragp;

	ip_addr_t ip_src;	/* Network order */
	ip_addr_t ip_dst;	/* Network order */
	u_short ip_id;		/* Network order */

	u_short maxlen;		
	u_short hadlastpacket;
};

#define IPFRAG_TIMEOUT		30

#define IPFRAG_MAX_MEM		(25*1024*1024)
#define IPFRAG_MAX_FRAGS	(10000)

void ip_fragment_init(void);
int ip_fragment(struct template *, struct ip_hdr *, u_short,
    struct ip_hdr **, u_short *);
void ip_send_fragments(u_int, struct ip_hdr *, u_int);
#endif
